package com.controller.common;

import cn.com.ty.lift.common.aliservice.aliyunoss.AliyunOSS;
import cn.com.ty.lift.common.verify.Verifier;
import cn.hutool.core.util.StrUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.aop.support.annotation.AnnotationMatchingPointcut;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;

/**
 * <p>
 * 配置bean注入
 * </p>
 *
 * @author wcz
 * @since 2019/12/3
 */
@Slf4j
@Data
@Configuration
@ConfigurationProperties(prefix = "aliyun.oss")
public class SystemConfiguration {

    private String endpoint;

    private String bucketName;

    private String accessKeySecret;

    private String accessKeyId;

    @PostConstruct
    private void print() {
        log.info("Hint: the Configuration for Aliyun OSS Client.");
        log.info("============================================================");
        log.info("###| endpoint       : {}", endpoint);
        log.info("###| accessKeyId    : {}", accessKeyId);
        log.info("###| accessKeySecret: {}", accessKeySecret);
        log.info("###| bucketName     : {}", bucketName);
        log.info("============================================================");
    }

    /**
     * 创建aliyun OSS bean,每次使用完连接释放
     */
    public AliyunOSS build() {
        if (StrUtil.hasEmpty(endpoint, accessKeyId, accessKeySecret, bucketName)) {
            return AliyunOSS.me();
        }
        return AliyunOSS.me(endpoint, accessKeyId, accessKeySecret, bucketName);
    }

    /**
     * 方法参数校验拦截器,标注了@Verifier的方法，会检查参数，如果参数不满足条件，直接抛出异常
     */
    @Bean
    public DefaultPointcutAdvisor defaultPointcutAdvisor() {
        DefaultPointcutAdvisor advisor = new DefaultPointcutAdvisor();
        advisor.setPointcut(new AnnotationMatchingPointcut(RestController.class, Verifier.class));
        advisor.setAdvice(new VerifyMethodInterceptor());
        advisor.setOrder(2);
        return advisor;
    }
}
